﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using SmartIdServer.Crypto;

namespace SmartIdServer.API {
    [Serializable]
    internal class Request {
        /// <summary>
        ///     Команда API
        /// </summary>
        [JsonConverter( typeof( StringEnumConverter ) )]
        public Commands Command { get; set; }

        /// <summary>
        ///     Сертифікат шифрування клієнта в кодуванні Base64
        /// </summary>
        public string Cert { get; set; }

        /// <summary>
        ///     Зашифрований запит в кодуванні Base64
        /// </summary>
        public string Encrypted { get; set; }
    }

    [Serializable]
    internal class Response {
        public Response() {
        }

        public Response( ErrorCode ec, string em = null ) {
            ErrorCode = ec;
            ErrorMessage = em ?? ec.ErrorMessage();
        }

        public Response( Response r ) {
            ErrorCode = r.ErrorCode;
            ErrorMessage = r.ErrorMessage;
        }

        public Response( DataError ex ) {
            ErrorCode = ex.Code;
            ErrorMessage = ex.Message;
        }

        /// <summary>
        ///     Код помилки
        /// </summary>
        [JsonConverter( typeof( StringEnumConverter ) )]
        public ErrorCode ErrorCode { get; set; } = ErrorCode.Ok;

        /// <summary>
        ///     Опис помилки
        /// </summary>
        public string ErrorMessage { get; set; }

        /// <summary>
        ///     Зашифрована відповідь в кодуванні Base64
        /// </summary>
        public string Encrypted { get; set; }
    }

    /// <summary>
    ///     Результат криптографічної операції
    /// </summary>
    [Serializable]
    internal class CryptOperResult {
        public CryptOperResult() {
        }

        public CryptOperResult( CryptographyErrorInternal ex ) {
            CryptResult = ex.CryptResult;
            CryptMessage = ex.Message;
        }

        /// <summary>
        ///     Код помилки
        /// </summary>
        public int CryptResult { get; set; } = (int)Crypto.CryptResult.CSP_RES_SUCCESS;

        /// <summary>
        ///     Опис помилки
        /// </summary>
        public string CryptMessage { get; set; }
    }

    /// <summary>
    ///     Запит стану сервера - запит
    /// </summary>
    [Serializable]
    internal class RequestServerState : Request {
        [OnSerializing]
        internal void OnSerializingMethod( StreamingContext context ) {
            Command = Commands.ServerState;
        }
    }

    /// <summary>
    ///     Запит стану сервера - відповідь
    /// </summary>
    [Serializable]
    internal class ResponseServerState : Response {
        public ResponseServerState() {
        }

        public ResponseServerState( ErrorCode ec ) : base( ec ) {
        }

        public ResponseServerState( ErrorCode ec, string em ) : base( ec, em ) {
        }

        public ResponseServerState( Response r ) : base( r ) {
        }

        public ResponseServerState( DataError ex ) : base( ex ) {
        }

        /// <summary>
        ///     Поточний час сервера
        /// </summary>
        public DateTime Timestamp { get; set; }
    }

    /// <summary>
    ///     Запит сертифікату шифрування сервера - запит
    /// </summary>
    [Serializable]
    internal class RequestGetEncryptCert : Request {
        [OnSerializing]
        internal void OnSerializingMethod( StreamingContext context ) {
            Command = Commands.GetEncryptCert;
        }
    }

    /// <summary>
    ///     Запит сертифікату шифрування сервера - відповідь
    /// </summary>
    [Serializable]
    internal class ResponseGetEncryptCert : Response {
        public ResponseGetEncryptCert() {
        }

        public ResponseGetEncryptCert( ErrorCode ec ) : base( ec ) {
        }

        public ResponseGetEncryptCert( ErrorCode ec, string em = null ) : base( ec, em ) {
        }

        public ResponseGetEncryptCert( Response r ) : base( r ) {
        }

        public ResponseGetEncryptCert( DataError ex ) : base( ex ) {
        }

        /// <summary>
        ///     Сертифікат шифрування сервера в кодуванні Base64
        /// </summary>
        public string Data { get; set; }
    }

    /// <summary>
    ///     Запит на створення Заявки і секретних ключів (внутрішня для КНЕДП) - запит
    /// </summary>
    [Serializable]
    internal class RequestGenerateKeys : Request {
        /// <summary>
        ///     Дані Заявки 1
        /// </summary>
        public UaRequestInfo Request1 { get; set; }

        /// <summary>
        ///     Дані Заявки 2
        /// </summary>
        public UaRequestInfo Request2 { get; set; }

        /// <summary>
        ///     Схема підписання і асиметричного шифрування
        /// </summary>
        [DefaultValue( "DSTU4145_PB9" )]
        [JsonProperty( DefaultValueHandling = DefaultValueHandling.Populate )]
        [JsonConverter( typeof( StringEnumConverter ) )]
        public TenSigningScheme ParamsId { get; set; }

        /// <summary>
        ///     Пароль секретного ключа
        /// </summary>
        public string Password { get; set; }

        [OnSerializing]
        internal void OnSerializingMethod( StreamingContext context ) {
            Command = Commands.GenerateKeys;
        }
    }

    /// <summary>
    ///     Запит на створення Заявки і секретних ключів (внутрішня для КНЕДП) - відповідь
    /// </summary>
    [Serializable]
    internal class ResponseGenerateKeys : Response {
        public ResponseGenerateKeys() {
        }

        public ResponseGenerateKeys( ErrorCode ec ) : base( ec ) {
        }

        public ResponseGenerateKeys( ErrorCode ec, string em = null ) : base( ec, em ) {
        }

        public ResponseGenerateKeys( Response r ) : base( r ) {
        }

        public ResponseGenerateKeys( DataError ex ) : base( ex ) {
        }

        /// <summary>
        ///     Ідентифікатор запиту
        /// </summary>
        public long RequestId { get; set; }
    }

    /// <summary>
    ///     Дані Заявки на сертифікат
    /// </summary>
    [Serializable]
    internal class UaRequestInfo {
        /// <summary>
        ///     Дані суб'єкта
        /// </summary>
        public UaSubjectInfo Subject { get; set; }

        /// <summary>
        ///     серийный номер сертификата (Hex)
        /// </summary>
        public string KeyIdentifier { get; set; }

        /// <summary>
        ///     Алгоритм открытого ключа
        /// </summary>
        [JsonConverter( typeof( StringEnumConverter ) )]
        public TenPublicKeyAlgorithm PkAlg { get; set; }

        /// <summary>
        ///     Объектный идентификатор алгоритма открытого ключа
        /// </summary>
        public string PkAlgOid { get; set; }

        /// <summary>
        ///     идентификатор открытого ключа пользователя (Hex)
        /// </summary>
        public string SubjectKeyIdentifier { get; set; }

        /// <summary>
        ///     сертифікат повинний бути посиленим
        /// </summary>
        public bool IsQualified { get; set; }

        /// <summary>
        ///     максимальная сумма финансовой транзакции
        /// </summary>
        public bool HasLimitValue { get; set; }

        /// <summary>
        ///     currencyCode, if empty - no desire
        /// </summary>
        [DefaultValue( "UAH" )]
        [JsonProperty( DefaultValueHandling = DefaultValueHandling.Populate )]
        public string CurrencyCode { get; set; }

        /// <summary>
        ///     limits to (amount * 10^exponent) currency
        /// </summary>
        public int Amount { get; set; }

        /// <summary>
        ///     limits to (amount * 10^exponent) currency
        /// </summary>
        public int Exponent { get; set; }

        /// <summary>
        ///     использование ключа (kuUserKey - для пользователей, kuCA - подпись серт-в, kuCRL - подпись СОС)
        /// </summary>
        [JsonConverter( typeof( StringEnumConverter ) )]
        public TenKeyUsage KeyUsage { get; set; }

        /// <summary>
        ///     использование ключа расширенное
        /// </summary>
        [JsonConverter( typeof( StringEnumConverter ) )]
        public TenExtKeyUsage ExtKeyUsage { get; set; }

        /// <summary>
        ///     ограничения сертификата, true - серт-т ЦСК, false - остальные
        /// </summary>
        public bool BasicConstraints { get; set; }

        /// <summary>
        ///     приватный ключ згенерований у пристрої
        /// </summary>
        public bool IsKeyInDevice { get; set; }

        /// <summary>
        ///     массив OID расширенного использования ключа
        /// </summary>
        public string[] ExtKeyUsageExt { get; set; }
    }

    /// <summary>
    ///     Дані суб'єкта
    /// </summary>
    [Serializable]
    internal class UaSubjectInfo {
        /// <summary>
        ///     ЕДРПОУ
        /// </summary>
        public string Edrpou { get; set; }

        /// <summary>
        ///     ДРФО
        /// </summary>
        public string Drfo { get; set; }

        /// <summary>
        ///     реквизиты владельца (поле CommonName)
        /// </summary>
        public string Name { get; set; }

        /// <summary>
        ///     почта
        /// </summary>
        public string Email { get; set; }

        /// <summary>
        ///     должность (поле Title), для физлиц обязательно
        /// </summary>
        public string Title { get; set; }

        /// <summary>
        ///     почтовый индекс
        /// </summary>
        public string PostalCode { get; set; }

        /// <summary>
        ///     область (stateOrProvinceName)
        /// </summary>
        public string Obl { get; set; }

        /// <summary>
        ///     город (localityName)
        /// </summary>
        public string Rayon { get; set; }

        /// <summary>
        ///     адрес
        /// </summary>
        public string Address { get; set; }

        /// <summary>
        ///     телефон
        /// </summary>
        public string Tel { get; set; }

        /// <summary>
        ///     dnQualifier
        /// </summary>
        public string Qualifier { get; set; }

        /// <summary>
        ///     фамилия (Surname), для физлиц обязательно
        /// </summary>
        public string Surname { get; set; }

        /// <summary>
        ///     имя отчество (GivenName), для физлиц обязательно
        /// </summary>
        public string GivenName { get; set; }

        /// <summary>
        ///     серийный номер пользователя
        /// </summary>
        public string SerialNumber { get; set; }

        /// <summary>
        ///     организация для юрлиц обязательно
        /// </summary>
        public string OrganizationName { get; set; }

        /// <summary>
        ///     подразделение
        /// </summary>
        public string OrganizationalUnitName { get; set; }

        /// <summary>
        ///     Єдиний державний демографічний реєстр України
        /// </summary>
        public string Eddru { get; set; }
    }

    /// <summary>
    ///     Одержання результату запиту на створення Заявки і секретних ключів (внутрішня для КНЕДП) - запит
    /// </summary>
    [Serializable]
    internal class RequestGenerateKeysResult : Request {
        /// <summary>
        ///     Ідентифікатор запиту
        /// </summary>
        public long RequestId { get; set; }

        [OnSerializing]
        internal void OnSerializingMethod( StreamingContext context ) {
            Command = Commands.GenerateKeysResult;
        }
    }

    /// <summary>
    ///     Одержання результату запиту на створення Заявки і секретних ключів (внутрішня для КНЕДП) - відповідь
    /// </summary>
    [Serializable]
    internal class ResponseGenerateKeysResult : Response {
        public ResponseGenerateKeysResult() {
        }

        public ResponseGenerateKeysResult( ErrorCode ec ) : base( ec ) {
        }

        public ResponseGenerateKeysResult( ErrorCode ec, string em = null ) : base( ec, em ) {
        }

        public ResponseGenerateKeysResult( Response r ) : base( r ) {
        }

        public ResponseGenerateKeysResult( DataError ex ) : base( ex ) {
        }

        /// <summary>
        ///     Ім'я ключа (Ідентифікатор клієнта)
        /// </summary>
        public string KeyName { get; set; }

        /// <summary>
        ///     Заявка 1 в форматі PKCS10, в кодуванні Base64
        /// </summary>
        public string Request1P10 { get; set; }

        /// <summary>
        ///     Заявка 2 в форматі PKCS10, в кодуванні Base64
        /// </summary>
        public string Request2P10 { get; set; }

        /// <summary>
        ///     Ідентифікатор ключа суб'єкта заявки 1 в форматі Hex
        /// </summary>
        public string SubjectKeyIdentifier1 { get; set; }

        /// <summary>
        ///     Ідентифікатор ключа суб'єкта заявки 2 в форматі Hex
        /// </summary>
        public string SubjectKeyIdentifier2 { get; set; }
    }

    /// <summary>
    ///     Запит на зміну пароля секретних ключів - запит
    /// </summary>
    [Serializable]
    internal class RequestChangeKeysPassword : Request {
        /// <summary>
        ///     Ім'я ключа (Ідентифікатор клієнта)
        /// </summary>
        public string KeyName { get; set; }

        /// <summary>
        ///     Ідентифікатор ключа суб'єкта сертифіката для одного з ключів в форматі Hex
        /// </summary>
        public string SubjectKeyIdentifier { get; set; }

        /// <summary>
        ///     Поточний пароль секретного ключа
        /// </summary>
        public string CurrentPassword { get; set; }

        /// <summary>
        ///     Новий пароль секретного ключа
        /// </summary>
        public string NewPassword { get; set; }

        [OnSerializing]
        internal void OnSerializingMethod( StreamingContext context ) {
            Command = Commands.ChangeKeysPassword;
        }
    }

    /// <summary>
    ///     Запит на зміну пароля секретних ключів - відповідь
    /// </summary>
    [Serializable]
    internal class ResponseChangeKeysPassword : Response {
        public ResponseChangeKeysPassword() {
        }

        public ResponseChangeKeysPassword( ErrorCode ec ) : base( ec ) {
        }

        public ResponseChangeKeysPassword( ErrorCode ec, string em = null ) : base( ec, em ) {
        }

        public ResponseChangeKeysPassword( Response r ) : base( r ) {
        }

        public ResponseChangeKeysPassword( DataError ex ) : base( ex ) {
        }

        /// <summary>
        ///     Ідентифікатор запиту
        /// </summary>
        public long RequestId { get; set; }
    }

    /// <summary>
    ///     Одержання результату запиту на зміну пароля секретних ключів - запит
    /// </summary>
    [Serializable]
    internal class RequestChangeKeysPasswordResult : Request {
        /// <summary>
        ///     Ідентифікатор запиту
        /// </summary>
        public long RequestId { get; set; }

        [OnSerializing]
        internal void OnSerializingMethod( StreamingContext context ) {
            Command = Commands.ChangeKeysPasswordResult;
        }
    }

    /// <summary>
    ///     Одержання результату запиту на зміну пароля секретних ключів - відповідь
    /// </summary>
    [Serializable]
    internal class ResponseChangeKeysPasswordResult : Response {
        public ResponseChangeKeysPasswordResult() {
        }

        public ResponseChangeKeysPasswordResult( ErrorCode ec ) : base( ec ) {
        }

        public ResponseChangeKeysPasswordResult( ErrorCode ec, string em = null ) : base( ec, em ) {
        }

        public ResponseChangeKeysPasswordResult( Response r ) : base( r ) {
        }

        public ResponseChangeKeysPasswordResult( DataError ex ) : base( ex ) {
        }
    }

    /// <summary>
    ///     Запит параметрів ключів - запит
    /// </summary>
    [Serializable]
    internal class RequestGetKeysParameters : Request {
        /// <summary>
        ///     Ім'я ключа (Ідентифікатор клієнта)
        /// </summary>
        public string KeyName { get; set; }

        /// <summary>
        ///     Ідентифікатор ключа суб'єкта сертифіката для одного з ключів в форматі Hex
        /// </summary>
        public string SubjectKeyIdentifier { get; set; }

        /// <summary>
        ///     Включати сертифікати у відповідь
        /// </summary>
        public bool IncludeCertificates { get; set; }

        [OnSerializing]
        internal void OnSerializingMethod( StreamingContext context ) {
            Command = Commands.GetKeysParameters;
        }
    }

    /// <summary>
    ///     Запит параметрів ключів - відповідь
    /// </summary>
    [Serializable]
    internal class ResponseGetKeysParameters : Response {
        public ResponseGetKeysParameters() {
        }

        public ResponseGetKeysParameters( ErrorCode ec ) : base( ec ) {
        }

        public ResponseGetKeysParameters( ErrorCode ec, string em = null ) : base( ec, em ) {
        }

        public ResponseGetKeysParameters( Response r ) : base( r ) {
        }

        public ResponseGetKeysParameters( DataError ex ) : base( ex ) {
        }

        /// <summary>
        ///     Ім'я ключа (Ідентифікатор клієнта)
        /// </summary>
        public string KeyName { get; set; }

        /// <summary>
        ///     Параметри ключів
        /// </summary>
        public KeyParameters[] Keys { get; set; }
    }

    /// <summary>
    ///     Параметри ключа
    /// </summary>
    [Serializable]
    internal class KeyParameters {
        /// <summary>
        ///     Ідентифікатор ключа суб'єкта сертифіката в форматі Hex
        /// </summary>
        public string SubjectKeyIdentifier { get; set; }

        /// <summary>
        ///     Сертифікат використовується для підписання
        /// </summary>
        public bool ForSign { get; set; }

        /// <summary>
        ///     Сертифікат використовується для шифрування
        /// </summary>
        public bool ForCrypt { get; set; }

        /// <summary>
        ///     Сертифікат в кодуванні Base64
        /// </summary>
        public string Cert { get; set; }
    }

    /// <summary>
    ///     Запит на виконання криптографічних операцій - запит
    /// </summary>
    [Serializable]
    internal class RequestDoOperations : Request {
        /// <summary>
        ///     Дані для підписання (Ідентифікатор ключа суб'єкта сертифіката для підписання в форматі Hex,
        ///     Дані для підписання)
        /// </summary>
        public Dictionary<string, OperSetSignature> SetSignatures { get; set; }

        /// <summary>
        ///     Дані для виконання запиту (Ідентифікатор ключа суб'єкта сертифіката для виконання запиту в форматі Hex,
        ///     Дані для виконання запиту)
        /// </summary>
        public Dictionary<string, OperRemoteRequest> RemoteRequests { get; set; }

        /// <summary>
        ///     Опис системи, що надсилає запит (для відображення користувачу під час підтвердження операції)
        /// </summary>
        public string DescriptionOriginator { get; set; }

        /// <summary>
        ///     Опис запиту (для відображення користувачу під час підтвердження операції)
        /// </summary>
        public string DescriptionOperation { get; set; }

        /// <summary>
        ///     Описи документів, що оброблюються у запиті (для відображення користувачу під час підтвердження операції)
        /// </summary>
        public string[] DescriptionDocuments { get; set; }

        [OnSerializing]
        internal void OnSerializingMethod( StreamingContext context ) {
            Command = Commands.DoOperations;
        }
    }

    /// <summary>
    ///     Запит на виконання криптографічних операцій - відповідь
    /// </summary>
    [Serializable]
    internal class ResponseDoOperations : Response {
        public ResponseDoOperations() {
        }

        public ResponseDoOperations( ErrorCode ec ) : base( ec ) {
        }

        public ResponseDoOperations( ErrorCode ec, string em = null ) : base( ec, em ) {
        }

        public ResponseDoOperations( Response r ) : base( r ) {
        }

        public ResponseDoOperations( DataError ex ) : base( ex ) {
        }

        /// <summary>
        ///     Ідентифікатор запиту
        /// </summary>
        public long RequestId { get; set; }
    }

    /// <summary>
    ///     Дані операції
    /// </summary>
    [Serializable]
    internal abstract class Oper {
        /// <summary>
        ///     Пароль секретного ключа
        /// </summary>
        public string Password { get; set; }
    }

    /// <summary>
    ///     Дані для підписання
    /// </summary>
    [Serializable]
    internal class OperSetSignature : Oper {
        /// <summary>
        ///     Дані для підписання (Ідентифікатор операції, Дані для підписання)
        /// </summary>
        public Dictionary<int, OperSetSignatureData> SetSignaturesData { get; set; }
    }

    /// <summary>
    ///     Дані для підписання
    /// </summary>
    [Serializable]
    internal class OperSetSignatureData {
        /// <summary>
        ///     Геш для підписання в кодуванні Base64
        /// </summary>
        public string Hash { get; set; }
    }

    /// <summary>
    ///     Дані для виконання запиту
    /// </summary>
    [Serializable]
    internal class OperRemoteRequest : Oper {
        /// <summary>
        ///     Дані для виконання запиту (Ідентифікатор операції, Дані для виконання запиту)
        /// </summary>
        public Dictionary<int, OperRemoteRequestData> RemoteRequestData { get; set; }
    }

    /// <summary>
    ///     Дані для виконання запиту
    /// </summary>
    [Serializable]
    internal class OperRemoteRequestData {
        /// <summary>
        ///     Дані для виконання запиту в кодуванні Base64
        /// </summary>
        public string Data { get; set; }
    }

    /// <summary>
    ///     Одержання результату запиту на виконання криптографічних операцій - запит
    /// </summary>
    [Serializable]
    internal class RequestDoOperationsResult : Request {
        /// <summary>
        ///     Ідентифікатор запиту
        /// </summary>
        public long RequestId { get; set; }

        /// <summary>
        ///     Ідентифікатори ключа суб'єкта сертифікатів, залучених до запиту на виконання криптографічних операцій,
        ///     в форматі Hex
        /// </summary>
        public string[] SubjectKeyIds { get; set; }

        [OnSerializing]
        internal void OnSerializingMethod( StreamingContext context ) {
            Command = Commands.DoOperationsResult;
        }
    }

    /// <summary>
    ///     Одержання результату запиту на виконання криптографічних операцій - відповідь
    /// </summary>
    [Serializable]
    internal class ResponseDoOperationsResult : Response {
        public ResponseDoOperationsResult() {
        }

        public ResponseDoOperationsResult( ErrorCode ec ) : base( ec ) {
        }

        public ResponseDoOperationsResult( ErrorCode ec, string em = null ) : base( ec, em ) {
        }

        public ResponseDoOperationsResult( Response r ) : base( r ) {
        }

        public ResponseDoOperationsResult( DataError ex ) : base( ex ) {
        }

        /// <summary>
        ///     Результати підписання (Ідентифікатор ключа суб'єкта сертифіката для підписання в форматі Hex,
        ///     Результати підписання)
        /// </summary>
        public Dictionary<string, OperSetSignatureResult> SetSignatures { get; set; }

        /// <summary>
        ///     Результати виконання запиту (Ідентифікатор ключа суб'єкта сертифіката для виконання запиту
        ///     в форматі Hex, Результати виконання запиту)
        /// </summary>
        public Dictionary<string, OperRemoteRequestResult> RemoteRequest { get; set; }
    }

    /// <summary>
    ///     Результати підписання
    /// </summary>
    [Serializable]
    internal class OperSetSignatureResult : Response {
        public OperSetSignatureResult() {
        }

        public OperSetSignatureResult( ErrorCode ec ) : base( ec ) {
        }

        public OperSetSignatureResult( ErrorCode ec, string em = null ) : base( ec, em ) {
        }

        public OperSetSignatureResult( Response r ) : base( r ) {
        }

        public OperSetSignatureResult( DataError ex ) : base( ex ) {
        }

        /// <summary>
        ///     Результати підписання (Ідентифікатор операції, Результат підписання)
        /// </summary>
        public Dictionary<int, OperSetSignatureResultData> SignData { get; set; }
    }

    /// <summary>
    ///     Результат підписання
    /// </summary>
    [Serializable]
    internal class OperSetSignatureResultData : CryptOperResult {
        public OperSetSignatureResultData() {
        }

        public OperSetSignatureResultData( CryptographyErrorInternal ex ) : base( ex ) {
        }

        /// <summary>
        ///     Підпис в кодуванні Base64
        /// </summary>
        public string Signature { get; set; }
    }

    /// <summary>
    ///     Результати виконання запиту
    /// </summary>
    [Serializable]
    internal class OperRemoteRequestResult : Response {
        public OperRemoteRequestResult() {
        }

        public OperRemoteRequestResult( ErrorCode ec ) : base( ec ) {
        }

        public OperRemoteRequestResult( ErrorCode ec, string em = null ) : base( ec, em ) {
        }

        public OperRemoteRequestResult( Response r ) : base( r ) {
        }

        public OperRemoteRequestResult( DataError ex ) : base( ex ) {
        }

        /// <summary>
        ///     Результати виконання запиту (Ідентифікатор операції, Результат виконання запиту)
        /// </summary>
        public Dictionary<int, OperRemoteRequestResultData> RemoteRequestData { get; set; }
    }

    /// <summary>
    ///     Результат виконання запиту
    /// </summary>
    [Serializable]
    internal class OperRemoteRequestResultData : CryptOperResult {
        public OperRemoteRequestResultData() {
        }

        public OperRemoteRequestResultData( CryptographyErrorInternal ex ) : base( ex ) {
        }

        /// <summary>
        ///     Дані результату виконання запиту в кодуванні Base64
        /// </summary>
        public string RemoteRequestResponse { get; set; }
    }

    /// <summary>
    ///     Одержання операцій, що вимагають підтвердження користувачем - запит
    /// </summary>
    [Serializable]
    internal class RequestGetOperationsForConfirmation : Request {
        /// <summary>
        ///     Імена ключів (Ідентифікатори клієнтів), які можуть бути підтверджені
        /// </summary>
        public string[] KeyNames { get; set; }

        [OnSerializing]
        internal void OnSerializingMethod( StreamingContext context ) {
            Command = Commands.GetOperationsForConfirmation;
        }
    }

    /// <summary>
    ///     Одержання операцій, що вимагають підтвердження користувачем - відповідь
    /// </summary>
    [Serializable]
    internal class ResponseGetOperationsForConfirmation : Response {
        public ResponseGetOperationsForConfirmation() {
        }

        public ResponseGetOperationsForConfirmation( ErrorCode ec ) : base( ec ) {
        }

        public ResponseGetOperationsForConfirmation( ErrorCode ec, string em = null ) : base( ec, em ) {
        }

        public ResponseGetOperationsForConfirmation( Response r ) : base( r ) {
        }

        public ResponseGetOperationsForConfirmation( DataError ex ) : base( ex ) {
        }

        /// <summary>
        ///     Операції, що вимагають підтвердження користувачем (Ідентифікатор запиту, Реквізити операції)
        /// </summary>
        public Dictionary<long, OperationForConfirmation> Operations { get; set; }
    }

    /// <summary>
    ///     Реквізити операції, що вимагає підтвердження користувачем
    /// </summary>
    [Serializable]
    internal class OperationForConfirmation {
        /// <summary>
        ///     Імена ключів (Ідентифікатори клієнтів)
        /// </summary>
        public string[] KeyNames { get; set; }

        /// <summary>
        ///     Опис системи, що надсилає запит (для відображення користувачу під час підтвердження операції)
        /// </summary>
        public string DescriptionOriginator { get; set; }

        /// <summary>
        ///     Опис запиту (для відображення користувачу під час підтвердження операції)
        /// </summary>
        public string DescriptionOperation { get; set; }

        /// <summary>
        ///     Описи документів, що оброблюються у запиті (для відображення користувачу під час підтвердження операції)
        /// </summary>
        public string[] DescriptionDocuments { get; set; }
    }

    /// <summary>
    ///     Підтвердження операцій користувачем - запит
    /// </summary>
    [Serializable]
    internal class RequestConfirmOperations : Request {
        /// <summary>
        ///     Імена ключів (Ідентифікатори клієнтів), які можуть бути підтверджені
        /// </summary>
        public string[] KeyNames { get; set; }

        /// <summary>
        ///     Ключі підтверджених операцій (Ім'я ключа, Пароль секретного ключа)
        /// </summary>
        public Dictionary<string, string> ConfirmedKeys { get; set; }

        /// <summary>
        ///     Підтверджені операції (Ідентифікатори запитів)
        /// </summary>
        public long[] Confirmed { get; set; }

        /// <summary>
        ///     Скасовані операції (Ідентифікатори запитів)
        /// </summary>
        public long[] Rejected { get; set; }

        [OnSerializing]
        internal void OnSerializingMethod( StreamingContext context ) {
            Command = Commands.ConfirmOperations;
        }
    }

    /// <summary>
    ///     Підтвердження операцій користувачем - відповідь
    /// </summary>
    [Serializable]
    internal class ResponseConfirmOperations : Response {
        public ResponseConfirmOperations() {
        }

        public ResponseConfirmOperations( ErrorCode ec ) : base( ec ) {
        }

        public ResponseConfirmOperations( ErrorCode ec, string em = null ) : base( ec, em ) {
        }

        public ResponseConfirmOperations( Response r ) : base( r ) {
        }

        public ResponseConfirmOperations( DataError ex ) : base( ex ) {
        }
    }

    /// <summary>
    ///     Підтвердження операції користувачем за QR-кодом (протокол IIT) - запит
    /// </summary>
    [Serializable]
    internal class RequestConfirmByQrCode : Request {
        /// <summary>
        ///     Ідентифікатор клієнта (із QR-коду)
        /// </summary>
        public string ClientId { get; set; }

        /// <summary>
        ///     Ідентифікатор операції (із QR-коду)
        /// </summary>
        public string OperationId { get; set; }

        /// <summary>
        ///     Ім'я ключа (Ідентифікатор клієнта)
        /// </summary>
        public string KeyName { get; set; }

        /// <summary>
        ///     Пароль секретного ключа (null - операцію скасовано)
        /// </summary>
        public string Password { get; set; }

        [OnSerializing]
        internal void OnSerializingMethod( StreamingContext context ) {
            Command = Commands.ConfirmByQrCode;
        }
    }

    /// <summary>
    ///     Підтвердження операції користувачем за QR-кодом (протокол IIT) - відповідь
    /// </summary>
    [Serializable]
    internal class ResponseConfirmByQrCode : Response {
        public ResponseConfirmByQrCode() {
        }

        public ResponseConfirmByQrCode( ErrorCode ec ) : base( ec ) {
        }

        public ResponseConfirmByQrCode( ErrorCode ec, string em = null ) : base( ec, em ) {
        }

        public ResponseConfirmByQrCode( Response r ) : base( r ) {
        }

        public ResponseConfirmByQrCode( DataError ex ) : base( ex ) {
        }
    }
}
