﻿using System;

namespace SmartIdClient {
    /// <summary>
    ///     Команди API
    /// </summary>
    internal enum Commands {
        /// <summary>
        ///     Запит стану сервера
        /// </summary>
        ServerState,

        /// <summary>
        ///     Запит сертифікату шифрування сервера
        /// </summary>
        GetEncryptCert,

        /// <summary>
        ///     Запит на створення Заявки і секретних ключів (внутрішня для КНЕДП)
        /// </summary>
        GenerateKeys,

        /// <summary>
        ///     Одержання результату запиту на створення Заявки і секретних ключів (внутрішня для КНЕДП)
        /// </summary>
        GenerateKeysResult,

        /// <summary>
        ///     Запит на виконання криптографічних операцій
        /// </summary>
        DoOperations,

        /// <summary>
        ///     Одержання результату запиту на виконання криптографічних операцій
        /// </summary>
        DoOperationsResult
    }

    /// <summary>
    ///     Код помилки
    /// </summary>
    internal enum ErrorCode {
        /// <summary>
        ///     OK
        /// </summary>
        Ok = 0,

        /// <summary>
        ///     Триває опрацювання запиту
        /// </summary>
        RequestProcessing = 1,

        /// <summary>
        ///     Помилка криптографічних функцій
        /// </summary>
        CrytographyError = 2,

        /// <summary>
        ///     Запит не зареєстрований
        /// </summary>
        RequestNotFound = 3,

        /// <summary>
        ///     Помилка валідації даних
        /// </summary>
        DataValidationError = 4,

        /// <summary>
        ///     Некоректний параметр запиту
        /// </summary>
        InvalidQueryParameter = 5,

        /// <summary>
        ///     Відсутній доступ
        /// </summary>
        AccessNotGranted = 6,

        /// <summary>
        ///     Сертифікат не зареєстрований
        /// </summary>
        CertificateNotFound = 7,

        /// <summary>
        ///     Приватний ключ не зареєстрований
        /// </summary>
        PkivateKeyNotFound = 8,

        /// <summary>
        ///     Некоректний пароль приватного ключа
        /// </summary>
        InvalidPrivateKeyPassword = 9
    }

    internal static class ErrorHelper {
        public static string ErrorMessage( this ErrorCode code ) {
            return code switch {
                ErrorCode.Ok => "Ok",
                ErrorCode.RequestProcessing => "Триває опрацювання запиту",
                ErrorCode.CrytographyError => "Помилка криптографічних функцій",
                ErrorCode.RequestNotFound => "Запит не зареєстрований",
                ErrorCode.DataValidationError => "Помилка валідації даних",
                ErrorCode.InvalidQueryParameter => "Некоректний параметр запиту",
                ErrorCode.AccessNotGranted => "Відсутній доступ",
                ErrorCode.CertificateNotFound => "Сертифікат не зареєстрований",
                ErrorCode.PkivateKeyNotFound => "Приватний ключ не зареєстрований",
                ErrorCode.InvalidPrivateKeyPassword => "Некоректний пароль приватного ключа",
                _ => null
            };
        }
    }

    public enum CryptResult {
        /// <summary>
        ///     Успішно
        /// </summary>
        CSP_RES_SUCCESS = 0,

        /// <summary>
        ///     Буфер порожній
        /// </summary>
        CryptBufferEmpty = 1,

        /// <summary>
        ///     DLL не ініціалізовано
        /// </summary>
        CryptDllNotLoaded = 2,

        /// <summary>
        ///     Помилка отримання інформації з сертифікату
        /// </summary>
        CryptBadCert = 3,

        /// <summary>
        ///     Даний сертифікат не може використовуватися для виконання операції
        /// </summary>
        CryptCertNotAllowed = 4,

        /// <summary>
        ///     Відміна введення пароля
        /// </summary>
        CryptPwdCancelled = 5,

        /// <summary>
        ///     Не збігається пара сертифікат - секретний ключ
        /// </summary>
        CryptSkNotMatch = 6,

        /// <summary>
        ///     Некоректний формат секретного ключа
        /// </summary>
        CryptSkCorrupt = 7,

        /// <summary>
        ///     Помилка підпису/шифрування, можливо вказано невірний пароль
        /// </summary>
        CryptBadPassword = 8,

        /// <summary>
        ///     Некоректний формат буфера, відсутня ідент. сигнатура
        /// </summary>
        CryptBadBuffer = 9,

        /// <summary>
        ///     Некоректний формат буфера
        /// </summary>
        CryptBufferCorrupt = 10,

        /// <summary>
        ///     Невірний підпис
        /// </summary>
        CryptBadSign = 11,

        /// <summary>
        ///     Внутрішня помилка перевірки підпису
        /// </summary>
        CryptInternalErr = 12,

        /// <summary>
        ///     Помилка перевірки цілісності: буфер пошкоджено
        /// </summary>
        CryptBadCrc = 13,

        /// <summary>
        ///     Блоки підписів зашифрованого та незашифрованого документа не збігаються
        /// </summary>
        CryptSgnBlkDif = 14,

        /// <summary>
        ///     Помилка перевірки сертифіката кореневим сертифікатом АЦСК
        /// </summary>
        CryptChkRootCert = 15,

        /// <summary>
        ///     Помилка dongle
        /// </summary>
        CryptErrorToken = 16,

        /// <summary>
        ///     Функції одержання позначки часу<br />
        ///     алгоритм хешування не підтримується
        /// </summary>
        CryptTsAlgHash = 21,

        /// <summary>
        ///     Функції одержання позначки часу<br />
        ///     помилка кодування запиту
        /// </summary>
        CryptTsEncReq = 22,

        /// <summary>
        ///     Функції одержання позначки часу<br />
        ///     помилка кодування оболонки
        /// </summary>
        CryptTsEncShell = 23,

        /// <summary>
        ///     Функції одержання позначки часу<br />
        ///     немає хеш-коду
        /// </summary>
        CryptTsNoHash = 24,

        /// <summary>
        ///     Функції одержання позначки часу<br />
        ///     помилка сокета
        /// </summary>
        CryptTsSock = 25,

        /// <summary>
        ///     Функції одержання позначки часу<br />
        ///     помилка відповіді сервера
        /// </summary>
        CryptTsAnswer = 26,

        /// <summary>
        ///     Функції одержання позначки часу<br />
        ///     позначка часу не відповідає документу
        /// </summary>
        CryptTsDifhash = 27,

        CSP_RES_ERROR_PKCS11 = 0x10000
    }

    [Flags]
    public enum TenKeyUsage : uint {
        enKuNone = 0x0000, // Нет сведений
        enKuDecipherOnly = 0x8000, // Только расшифрование (80 00)
        enKuEncipherOnly = 0x0001, // Только зашифрование (01)
        enKuCRLSign = 0x0002, // Подписывание списка отзыва (CRL) (02)
        enKuKeyCertSign = 0x0004, // Подписывание сертификатов (04)
        enKuKeyAgreement = 0x0008, // Согласование ключей (08)
        enKuDataEncipherment = 0x0010, // Шифрование данных (10)
        enKuKeyEncipherment = 0x0020, // Шифрование ключей (20)
        enKuNonRepudiation = 0x0040, // Неотрекаемость (40)
        enKuDigitalSignature = 0x0080, // Цифровая подпись (80)
        enKuUnknown = 0x0100 // Неизвестное использование ключа (00 01)
    }

    /// <summary>
    ///     Использование ключа расширенное
    /// </summary>
    [Flags]
    public enum TenExtKeyUsage : uint {
        enEKuNone = 0x0000, // Нет сведений

        //enEKuDef         = 0x0001, // Сертификат пользователя, сертификат ЦСК (1.2.804.2.1.1.1.3.9)
        enEKuOCSP = 0x0002, // Сертификат сервера OCSP (статус сертификата) (1.3.6.1.5.5.7.3.9)
        enEKuTSP = 0x0004, // Сертификат сервера TSP (метка времени) (1.3.6.1.5.5.7.3.8)
        enEKuAuthServer = 0x0008, // Проверка подлинности сервера (1.3.6.1.5.5.7.3.1)
        enEKuAuthClient = 0x0010, // Проверка подлинности клиента (1.3.6.1.5.5.7.3.2)
        enEKuAuthCode = 0x0020, // Защита программных средств (1.3.6.1.5.5.7.3.3)
        enEKuAuthEmail = 0x0040, // Защита электронной почты (1.3.6.1.5.5.7.3.4)

        //enEKuOrgStamp    = 0x0080, // Печать организации (1.2.804.2.1.1.1.3.9)
        //enEKuRegClaim    = 0x0100, // Регистрация заявок (1.2.804.2.1.1.1.3.10)
        enEKuAnyUsage = 0x0200, // Любое использование ключа (2.5.29.37.0)

        enEKuCertTypeDir = 0x010000, // Директор (1.2.804.2.1.1.1.11.36865753.1)
        enEKuCertTypeBuhg = 0x020000, // Бухгалтер (1.2.804.2.1.1.1.11.36865753.2)
        enEKuCertTypeStamp = 0x040000, // Печать - 1.2.804.2.1.1.1.3.9
        enEKuCertTypeSgNlNk = 0x080000, // Подпись налоговых накладных - 1.2.804.2.1.1.1.11.36865753.3
        enEKuCertTypeAdmReg = 0x100000, // Администратор регистрации УСЦ - 1.2.804.2.1.1.1.11.36865753.4
        enEKuCertTypePhys = 0x200000, // Физическое лицо - 1.2.804.2.1.1.1.11.36865753.5

        enEKuCertTypes = 0xff0000 //
    }

    public enum TenPublicKeyAlgorithm : uint {
        enDSTU4145_ALG = 0, // ДСТУ-4145
        enRSA_ALG = 1, // RSA
        enECDSA_ALG = 2, // ECDSA
        enOTHER_ALG = 3 // Другая
    }

    /// <summary>
    ///     Схема подписания и асимметричного шифрования
    /// </summary>
    public enum TenSigningScheme : uint {
        DSTU4145_PB0 = 0,
        DSTU4145_PB1,
        DSTU4145_PB2,
        DSTU4145_PB3,
        DSTU4145_PB4,
        DSTU4145_PB5,
        DSTU4145_PB6,
        DSTU4145_PB7,
        DSTU4145_PB8,
        DSTU4145_PB9,

        MD2_RSA_1024,
        MD2_RSA_2048,

        MD4_RSA_1024,
        MD4_RSA_2048,

        MD5_RSA_1024,
        MD5_RSA_2048,

        SHA1_RSA_1024,
        SHA1_RSA_2048,
        SHA256_RSA_1024,
        SHA256_RSA_2048,
        SHA384_RSA_1024,
        SHA384_RSA_2048,
        SHA512_RSA_1024,
        SHA512_RSA_2048,

        OTHER_SCHEME,

        SHA256_ECDSA192 = 28,
        SHA256_ECDSA256 = 30,
        SHA256_ECDSA384 = 31,
        SHA256_ECDSA521 = 32,

        DSTU4145_KUPINA_PB0 = 40,
        DSTU4145_KUPINA_PB1,
        DSTU4145_KUPINA_PB2,
        DSTU4145_KUPINA_PB3,
        DSTU4145_KUPINA_PB4,
        DSTU4145_KUPINA_PB5,
        DSTU4145_KUPINA_PB6,
        DSTU4145_KUPINA_PB7,
        DSTU4145_KUPINA_PB8,
        DSTU4145_KUPINA_PB9,

        DSTU257_DKE4A92 = 101
    }
}
